#include <drivers/counter.h>
#include <drivers/spi.h>
#include <logging/log.h>

LOG_MODULE_REGISTER(counter_spim);

#define TIMER_NODE DT_NODELABEL(timer1)
#define SPI_NODE DT_NODELABEL(spi1)

static const struct device *timer_ = DEVICE_DT_GET(TIMER_NODE);
static const struct device *spi_ = DEVICE_DT_GET(SPI_NODE);
const struct spi_cs_control spi_cs_ = {.gpio_dev = DEVICE_DT_GET(DT_GPIO_CTLR(SPI_NODE, cs_gpios)),
                                       .gpio_pin = DT_GPIO_PIN(SPI_NODE, cs_gpios),
                                       .gpio_dt_flags = DT_GPIO_FLAGS(SPI_NODE, cs_gpios)};
const struct spi_config spi_conf_ = {
    .operation = SPI_OP_MODE_MASTER | SPI_WORD_SET(8) | SPI_LINES_SINGLE,
    .frequency = 1000000,
    .cs = &spi_cs_,
};

static uint8_t tx_buf_raw_[2];
static uint8_t rx_buf_raw_[2];

static const struct spi_buf tx_bufs_[] = {
    {
        .buf = tx_buf_raw_,
        .len = sizeof(tx_buf_raw_),
    },
};
static const struct spi_buf rx_bufs_[] = {
    {
        .buf = rx_buf_raw_,
        .len = sizeof(rx_buf_raw_),
    },
};

static const struct spi_buf_set tx_ = {.buffers = tx_bufs_, .count = ARRAY_SIZE(tx_bufs_)};
static const struct spi_buf_set rx_ = {.buffers = rx_bufs_, .count = ARRAY_SIZE(rx_bufs_)};

void counter_interrupt_fn(const struct device *counter_dev, void *user_data)
{
	int err = spi_transceive(spi_, &spi_conf_, &tx_, &rx_);
	if (err) {
		LOG_ERR("error: %d", err);
	}
}

void main(void)
{
	LOG_INF("boot");

    const struct counter_top_cfg timer_cfg = {
        .ticks = counter_us_to_ticks(timer_, 1000000),
        .callback = counter_interrupt_fn,
    };

    counter_set_top_value(timer_, &timer_cfg);
	counter_start(timer_);
}
